"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const styled_components_1 = require("styled-components");
// import { Hitting } from '../../Loading/Animation/styles';
const state_1 = require("./state");
// import SwipeableStyledListItem from './swipeableStyledListItem';
// import {
//   LeadingActions,
//   TrailingActions,
//   Type as ListType,
// } from 'react-swipeable-list';
const Animation_1 = __importDefault(require("../../Loading/Animation"));
const styles_1 = require("../../Loading/Animation/styles");
const styles_2 = require("./styles");
const scrollTo = (ref, to, jump, y) => {
    ref?.current?.scrollTo({
        left: y ? undefined : to,
        top: y ? to : undefined,
        behavior: jump ? 'auto' : 'smooth',
    });
};
class Item extends react_1.default.Component {
    itemRef;
    leadingRef;
    trailingRef;
    wrapperRef;
    animationRef;
    constructor(props) {
        super(props);
        this.itemRef = react_1.default.createRef();
        this.wrapperRef = react_1.default.createRef();
        this.animationRef = react_1.default.createRef();
        this.leadingRef = react_1.default.createRef();
        this.trailingRef = react_1.default.createRef();
        // console.log('vertical', props.vertical);
        this.state = {
            leading: this.passCallbacks(props.leading),
            trailing: this.passCallbacks(props.trailing),
            children: this.passCallbacksToChildren(props.children),
            start: undefined,
            current: undefined,
            vertical: props.vertical || false,
            state: state_1.State.NONE,
            index: undefined,
            fullSwipe: props.fullSwipe == undefined ? true : props.fullSwipe,
            threshold: props.threshold || 0.15,
            holdSwipeThreshold: props.holdSwipeThreshold || 0.1,
            fullSwipeThreshold: props.fullSwipeThreshold || 0.5,
            holdThreshold: props.holdThreshold || 0.5,
            holdAnimation: props.holdAnimation == undefined ? true : props.holdAnimation,
            holdTimeout: undefined,
            leadingPosition: 0,
            defaultPosition: 0,
            trailingPosition: 0,
            leadingSize: 0,
            trailingSize: 0,
            opositeSize: 0,
            destroyAnimationTime: props.destroyAnimationTime || 0.25,
        };
    }
    onEvent(event, currentState) {
        // console.log('onEvent', this.state.state, currentState);
        if (currentState === state_1.State.CLICK) {
            if (this.state.state === state_1.State.LEADING ||
                this.state.state === state_1.State.TRAILING) {
                this.setState({ state: state_1.State.NONE });
            }
            else {
                // console.log('e click');
                this.props.onClick?.(event);
            }
        }
        else if (currentState === state_1.State.HOLD_END) {
            // console.log('e hold end');
            this.props.onHoldEnd?.(event);
        }
        else {
            // scrollTo(this.animationRef, this.state.defaultPosition);
            // scrollTo(this.wrapperRef, this.state.defaultPosition);
            // console.log('e hold');
            this.props.onHold?.(event);
        }
    }
    onMouseMove(event) {
        return this.onMove(event, false);
    }
    onTouchMove(event) {
        return this.onMove(event, true);
    }
    getCurrentSize(currentState) {
        return currentState === state_1.State.LEADING
            ? this.state.leadingSize
            : currentState === state_1.State.TRAILING
                ? this.state.trailingSize
                : 0;
    }
    hasSwiped(swipe, threshold) {
        threshold = threshold || this.state.threshold;
        const currentState = this.getState(swipe);
        const walked = this.calcWalked(swipe);
        const currentSize = this.getCurrentSize(currentState);
        return ((currentState === state_1.State.LEADING || currentState === state_1.State.TRAILING) &&
            currentSize > 0 &&
            walked > threshold);
    }
    calcWalked(swipe) {
        swipe = swipe || 0;
        let currentState = this.getState(swipe);
        const lengthWalked = Math.abs(swipe);
        const lengthTotal = (currentState === state_1.State.LEADING
            ? this.state.leadingSize
            : this.state.trailingSize) || lengthWalked;
        const walked = lengthWalked / lengthTotal;
        return walked;
    }
    calcSwipe(start, current) {
        start = start || 0;
        current = current == undefined ? start : current;
        const swipe = current - start;
        // console.log('calcSwipe', start, current);
        return swipe;
    }
    getState(swipe) {
        let currentState;
        if (swipe == undefined || swipe === 0) {
            currentState = state_1.State.NONE;
        }
        else if (swipe > 0) {
            currentState = state_1.State.LEADING;
        }
        else if (swipe < 0) {
            currentState = state_1.State.TRAILING;
        }
        return currentState;
    }
    isOverLeading(swipe) {
        swipe = swipe || 0;
        let currentState = this.getState(swipe);
        return (this.state.leadingSize > 0 &&
            currentState === state_1.State.LEADING &&
            Math.abs(swipe) > this.state.leadingSize);
    }
    isOverTrailing(swipe) {
        swipe = swipe || 0;
        let currentState = this.getState(swipe);
        return (this.state.trailingSize > 0 &&
            currentState === state_1.State.TRAILING &&
            Math.abs(swipe) > this.state.leadingSize);
    }
    resetStyles() {
        if (this.leadingRef?.current?.style) {
            this.leadingRef.current.style.width = `${this.state.leadingSize}px`;
        }
        if (this.trailingRef?.current?.style) {
            this.trailingRef.current.style.width = `${this.state.trailingSize}px`;
        }
        if (this.itemRef?.current?.style) {
            this.itemRef.current.style.transform = `translateX(${0}px)`;
        }
    }
    clearHold() {
        clearTimeout(this.state.holdTimeout);
        this.setState({ holdTimeout: undefined });
    }
    checkHold(swipe) {
        if (this.hasSwiped(swipe, this.state.holdSwipeThreshold)) {
            this.clearHold();
        }
    }
    onMove(event, _touch) {
        const current = this.state.vertical ? event.clientY : event.clientX;
        // console.log('onMove', current, this.state.start);
        const swipe = this.calcSwipe(this.state.start, current);
        if (this.state.start != undefined) {
            if (this.leadingRef?.current?.style && this.isOverLeading(swipe)) {
                this.leadingRef.current.style.width = `${Math.abs(swipe)}px`;
                if (this.itemRef?.current?.style)
                    this.itemRef.current.style.transform = `translateX(${Math.abs(swipe - this.state.leadingSize)}px)`;
            }
            if (this.trailingRef?.current?.style && this.isOverTrailing(swipe)) {
                this.trailingRef.current.style.width = `${Math.abs(swipe)}px`;
            }
            scrollTo(this.wrapperRef, this.state.defaultPosition - swipe, true);
        }
        this.checkHold(swipe);
        this.setState({ current });
    }
    onStart(event, touch) {
        // console.log('onStart', this.state.state);
        if (this.state.state === state_1.State.HOLD) {
            this.setState({ state: state_1.State.HOLD_END });
        }
        else {
            const start = this.state.vertical ? event.clientY : event.clientX;
            this.addMove(touch);
            const holdTimeout = setTimeout(() => {
                this.setState({ state: state_1.State.HOLD });
            }, this.state.holdThreshold * 1000);
            this.setState({ start, holdTimeout });
        }
        // this.addLeave(touch);
    }
    onEnd(event, touch) {
        this.clearHold();
        if (this.state.state === state_1.State.HOLD_END) {
            this.setState({
                state: state_1.State.NONE,
                start: undefined,
                current: undefined,
            });
            this.onEvent(event, state_1.State.HOLD_END);
        }
        else {
            const swipe = this.calcSwipe(this.state.start, this.state.current);
            let currentState = this.getState(swipe);
            const walked = this.calcWalked(swipe);
            const swiped = this.hasSwiped(swipe);
            // console.log('onEnd', swipe, walked);
            if (swiped) {
                // console.log('onEnd', swiped);
                if (this.state.fullSwipe && walked > this.state.fullSwipeThreshold) {
                    this.onFullSwiped(event, currentState);
                }
                else {
                    this.onSwiped(event, currentState);
                }
            }
            else {
                // console.log('onEnd close', swiped);
                this.close();
                if (this.state.state === state_1.State.HOLD) {
                    currentState = state_1.State.HOLD;
                }
                else {
                    currentState = state_1.State.CLICK;
                }
                this.onEvent(event, currentState);
            }
            this.setState({
                state: currentState,
                start: undefined,
                current: undefined,
            });
        }
        this.resetStyles();
        this.removeMove(touch);
        // this.removeLeave(touch);
    }
    onSwiped(_event, state) {
        // console.log('onSwiped', event, state);
        switch (state) {
            case state_1.State.LEADING:
                scrollTo(this.wrapperRef, this.state.leadingPosition);
                break;
            case state_1.State.TRAILING:
                scrollTo(this.wrapperRef, this.state.trailingPosition);
                break;
        }
    }
    onFullSwiped(_event, state) {
        // console.log('onFullSwiped', event, state);
        switch (state) {
            case state_1.State.LEADING:
                const first = this.leadingRef?.current
                    ?.firstElementChild;
                first?.click();
                break;
            case state_1.State.TRAILING:
                const last = this.trailingRef?.current
                    ?.lastElementChild;
                last?.click();
                break;
        }
        // this.close();
    }
    close(_isButton) {
        // console.log('close', isButton);
        scrollTo(this.wrapperRef, this.state.defaultPosition);
    }
    destroy(_isButton) {
        // console.log('destroy', isButton);
        scrollTo(this.wrapperRef, this.state.trailingPosition);
        if (this.wrapperRef?.current)
            this.wrapperRef.current.style.maxHeight = '0px';
        setTimeout(() => {
            // console.log('destroy timeout');
            if (this.wrapperRef?.current)
                this.wrapperRef?.current?.remove();
        }, this.state.destroyAnimationTime * 1000);
    }
    passCallbacksToChildren(children) {
        const childrenWithProps = react_1.default.Children.map(children, (child) => {
            return child != undefined
                ? react_1.default.cloneElement(child, {
                    destroy: this.destroy.bind(this),
                    close: this.close.bind(this),
                })
                : child;
        });
        return childrenWithProps;
    }
    passCallbacks(block) {
        const childrenWithProps = react_1.default.Children.map(block?.props?.children, (child) => {
            const newChild = child != undefined
                ? react_1.default.cloneElement(child, {
                    destroy: this.destroy.bind(this),
                    close: this.close.bind(this),
                })
                : child;
            // console.log('child', child?.props, newChild?.props);
            return newChild;
        });
        // console.log('passCallbacks', childrenWithProps);
        return childrenWithProps;
    }
    onMouseLeave(event) {
        return this.onLeave.bind(this)(event, false);
    }
    onTouchLeave(event) {
        return this.onLeave.bind(this)(event, true);
    }
    onLeave = (_event, touch) => {
        // console.log('onLeave', event, touch);
        this.setState({ state: state_1.State.NONE });
        this.removeMove(touch);
    };
    onTouchMoveBinded = this.onTouchMove.bind(this);
    onMouseMoveBinded = this.onMouseMove.bind(this);
    onTouchLeaveBinded = this.onTouchLeave.bind(this);
    onMouseLeaveBinded = this.onMouseLeave.bind(this);
    addMove(touch) {
        this.wrapperRef?.current?.addEventListener(touch ? 'touchmove' : 'mousemove', touch ? this.onTouchMoveBinded : this.onMouseMoveBinded);
    }
    // public addLeave(touch?: boolean) {
    //   this.wrapperRef?.current?.addEventListener(
    //     touch ? 'touchcancel' : 'mouseleave',
    //     touch ? this.onTouchLeaveBinded : this.onMouseLeaveBinded
    //   );
    // }
    removeMove(touch) {
        this.wrapperRef?.current?.removeEventListener(touch ? 'touchmove' : 'mousemove', touch ? this.onTouchMoveBinded : this.onMouseMoveBinded);
    }
    // public removeLeave(touch?: boolean) {
    //   this.wrapperRef?.current?.removeEventListener(
    //     touch ? 'touchcancel' : 'mouseleave',
    //     touch ? this.onTouchLeaveBinded : this.onMouseLeaveBinded
    //   );
    // }
    item() {
        return (react_1.default.createElement(styles_2.Wrapper, { time: this.state.destroyAnimationTime, ref: this.wrapperRef, onMouseDown: (event) => this.onStart.bind(this)(event, false), onMouseUp: (event) => this.onEnd.bind(this)(event, false), 
            // onMouseLeave={this.onLeave}
            // onMouseMove={this.onMove}
            onTouchStart: (event) => this.onStart.bind(this)(event, true), onTouchEnd: (event) => this.onEnd.bind(this)(event, true), 
            // onTouchCancel={(event) => this.onLeave(event, true)}
            // onTouchMove={(event) => this.onMove(event, true)}
            className: this.state.vertical ? 'vertical' : 'horizontal' },
            react_1.default.createElement(styles_2.Item, { className: 'leading', ref: this.leadingRef }, this.state.leading),
            react_1.default.createElement(styles_2.Item, { className: 'main', ref: this.itemRef }, this.state.children),
            react_1.default.createElement(styles_2.Item, { className: 'trailing', ref: this.trailingRef }, this.state.trailing)));
    }
    componentDidMount() {
        const leadingSize = this.leadingRef?.current?.offsetWidth || 0;
        const trailingSize = this.trailingRef?.current?.offsetWidth || 0;
        const defaultPosition = leadingSize;
        const trailingPosition = defaultPosition + (this.itemRef?.current?.offsetWidth || 0);
        const opositeSize = this.itemRef?.current?.clientHeight || 0;
        this.setState({
            defaultPosition,
            trailingPosition,
            leadingSize,
            trailingSize,
            opositeSize,
        });
        if (this.wrapperRef?.current)
            this.wrapperRef.current.style.maxHeight = `${opositeSize}px`;
        scrollTo(this.wrapperRef, defaultPosition);
        // if (this.animationRef?.current) {
        //   this.animationRef.current.style.maxHeight = `${opositeSize}px`;
        //   this.animationRef.current.style.overflow = `hidden`;
        // }
        // scrollTo(this.animationRef, defaultPosition);
    }
    componentDidUpdate(_prevProps, prevState) {
        if (this.state.state != prevState.state ||
            (this.state.state == state_1.State.HOLD &&
                ((this.state.start != prevState.start &&
                    this.state.start == undefined) ||
                    (this.state.current != prevState.current &&
                        this.state.current == undefined)))) {
            // console.log('componentDidUpdate', this.state.state);
            if (this.state.state !== state_1.State.TRAILING &&
                this.state.state !== state_1.State.LEADING) {
                if (this.wrapperRef?.current)
                    this.wrapperRef.current.style.maxHeight = `${this.state.opositeSize}px`;
                scrollTo(this.wrapperRef, this.state.defaultPosition, this.state.state >= state_1.State.HOLD);
            }
        }
        // else {
        //   console.log(
        //     'componentDidUpdate',
        //     this.state.state,
        //     prevState.state,
        //     this.state.start,
        //     prevState.start,
        //     this.state.current,
        //     prevState.current
        //   );
        // }
    }
    render() {
        return this.state.holdAnimation && this.state.state === state_1.State.HOLD ? (react_1.default.createElement(Animation_1.default, { ref: this.animationRef, crude: true, Animation: (0, styled_components_1.withTheme)(styles_1.Hitting), style: { width: '100%' } }, this.item())) : (react_1.default.createElement(react_1.default.Fragment, null, this.item()));
    }
}
exports.default = (0, styled_components_1.withTheme)(Item);
//# sourceMappingURL=index.js.map